#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include "txtren.h"
#include "config.h"
#include "pngio.h"

cfg_button_maker_t cfg;

unsigned int *data;
int w,h;

struct { int w,x; uint8_t *data; } parts[200];

static void setpix(int x,int y, unsigned int color)
{
  if (x<0 || y<0 || x>=w || y>=h) return ;
  data[x+y*w]= color | (0xff<<24);
}

static unsigned int combine(unsigned char K, unsigned int fg, unsigned int bg)
{
  double V;
  unsigned int Rf,Gf,Bf, Rb,Gb,Bb, R,G,B, P;
  V= K;
  V/= 255.0;

  Rf= (fg>>16)&0xff; Gf= (fg>>8) &0xff; Bf= fg      &0xff;
  Rb= (bg>>16)&0xff; Gb= (bg>>8) &0xff; Bb= bg      &0xff;

  R= Rf*V + Rb*(1.0-V);
  G= Gf*V + Gb*(1.0-V);
  B= Bf*V + Bb*(1.0-V);

  P= (R&0xff)<<16;
  P|= (G&0xff)<<8;
  P|= B&0xff;
  return P;
}

int vertline(int Xp)
{
  int x,y;
  for(y=0;y<h;y++)
   for(x=0;x<cfg.border_size;x++)
     setpix(x+Xp, y, cfg.border_color);
  return cfg.border_size;
}

static void usage(char **argv)
{
  fprintf(stderr,"usage: %s config-file output.png "
                        "text1 text2 ..\n", argv[0]);
  fprintf(stderr,"\n");
  fprintf(stderr,"The config file can have the following entries:\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"   int font_size;\n");
  fprintf(stderr,"   string font_file;\n");
  fprintf(stderr,"   int vertical_padding;\n");
  fprintf(stderr,"   int horizontal_padding;\n");
  fprintf(stderr,"   int border_size;\n");
  fprintf(stderr,"   int background;\n");
  fprintf(stderr,"   int foreground;\n");
  fprintf(stderr,"   int border_color;\n");

  fprintf(stderr,"\nThe font_size entry is mandatory, others have defaults.\n");
  fprintf(stderr,"If your font is too big, try negative paddings.\n");
  exit(1);
}

int main(int argc,char **argv)
{
  int tw,fonth;
  char *cfn;
  char *e;
  u_font_t *font;
  int i,x,y,Xp,Yp;
  uint8_t *tdata;

  if (argc<4) usage(argv);

  cfn= argv[1];
  e= cfg_parse_button_maker(cfn, &cfg);
  if (e)
     exit(fprintf(stderr,"error parsing %s: %s\n",cfn,e));

  if (!cfg.G_font_file)
     usage(argv);

  if (!cfg.G_font_size) cfg.font_size= 10;
  if (!cfg.G_foreground) cfg.foreground= 0x000000;
  if (!cfg.G_background) cfg.background= 0xffffff;
  if (!cfg.G_border_color) cfg.border_color= 0x000000;
  if (!cfg.G_border_size) cfg.border_size= 2;
  if (!cfg.G_horizontal_padding) cfg.horizontal_padding= 2;
  if (!cfg.G_vertical_padding) cfg.vertical_padding= 2;

  font_init();

  font= font_new(cfg.font_file, cfg.font_size);
  if (!font)
    exit(fprintf(stderr,"can't open font file %s\n", cfg.font_file));

  fonth= font->height;
  
  Xp= cfg.border_size;

  for(i=3;i<argc;i++)
  {
    Xp+= cfg.horizontal_padding;
    parts[i].x= Xp;
    text_render(font,argv[i],&parts[i].w, &parts[i].data);

    Xp+= parts[i].w + cfg.horizontal_padding + cfg.border_size;
  }

  w= Xp;
  h= fonth + cfg.vertical_padding*2 + cfg.border_size*2;

  data= malloc(sizeof(unsigned int)*w*h);

  for(y=0;y<h;y++)
  for(x=0;x<w;x++)
    setpix(x,y, cfg.background);

  Yp= cfg.border_size + cfg.vertical_padding;
  for(i=3;i<argc;i++)
  {
    Xp= parts[i].x;
    for(y=0;y<fonth;y++)
      for(x=0;x<parts[i].w;x++)
        setpix(x+Xp, y+Yp, combine(parts[i].data[x+y*parts[i].w],
                                   cfg.foreground, cfg.background));
  }

  for(x=0;x<w;x++)
   for(y=0;y<cfg.border_size;y++)
   {
    setpix(x,y,  cfg.border_color);
    setpix(x,y+h-cfg.border_size,  cfg.border_color);
   }

  vertline(0);
  for(i=3;i<argc;i++)
    vertline(parts[i].x+parts[i].w+cfg.horizontal_padding);

  e= write_png_image(argv[2],w,h,data);
  if (e)
   exit(fprintf(stderr,"can't write %s: %s\n",argv[2],e));

  free(data);
  font_finish();

  return 0;
}
